#pragma once

#include "ImageProxy.h"

#include <GdiPlus.h>
class CGDIPlusImage : public IImage
{
public:
	CGDIPlusImage()
	{
		m_pBitmap = NULL;
		m_hBuffer = NULL;
	}
	~CGDIPlusImage()
	{
		if (m_hBuffer)
		{
			::GlobalUnlock(m_hBuffer);
			::GlobalFree(m_hBuffer);
			m_hBuffer = NULL;
		}
		if (m_pBitmap)
		{
			delete m_pBitmap;
			m_pBitmap = NULL;
		}
	}
	void DrawImage(HDC hDC, LPRECT lpSrcRect, LPRECT lpDestRect)
	{
		if (m_pBitmap == NULL) return;

		Gdiplus::Graphics g(hDC);
		g.SetPixelOffsetMode(Gdiplus::PixelOffsetModeHalf);
		g.SetInterpolationMode(Gdiplus::InterpolationModeNearestNeighbor);
		
		Gdiplus::Rect rcDest(lpDestRect->left, lpDestRect->top, abs(lpDestRect->right-lpDestRect->left), abs(lpDestRect->bottom-lpDestRect->top));
		CRect rcSrc(lpSrcRect);
		g.DrawImage(m_pBitmap, rcDest, rcSrc.left, rcSrc.top, rcSrc.Width(), rcSrc.Height(), Gdiplus::UnitPixel,
			NULL, NULL, NULL);
	}
	BOOL TransparentBlt(HDC hDestDC, LPRECT lpSrcRect, LPRECT lpDestRect, UINT crTransparent = CLR_INVALID)
	{
		DrawImage(hDestDC, lpSrcRect, lpDestRect);
		return TRUE;
	}
	int GetWidth()
	{
		if (m_pBitmap == NULL) return -1;
		return m_pBitmap->GetWidth();
	}
	int GetHeight()
	{
		if (m_pBitmap == NULL) return -1;
		return m_pBitmap->GetHeight();
	}

	BOOL LoadFromFile(LPCTSTR pszcFile)
	{
		USES_CONVERSION	;
		Gdiplus::Bitmap* pBitmap = Gdiplus::Bitmap::FromFile(CT2CW(pszcFile));
		if (pBitmap)
		{
			m_pBitmap = pBitmap;
			return TRUE;
		}
		return FALSE;
	}
	BOOL Load(LPBYTE pBuf, DWORD dwBufLen)
	{
		BOOL bSuccess = FALSE;
		m_hBuffer = ::GlobalAlloc(GMEM_MOVEABLE, dwBufLen);
		if (m_hBuffer)
		{
			void* pBuffer = ::GlobalLock(m_hBuffer);
			if (pBuffer)
			{
				CopyMemory(pBuffer, pBuf, dwBufLen);

				CComPtr<IStream> pStream = NULL;
				if (::CreateStreamOnHGlobal(m_hBuffer, TRUE, &pStream) == S_OK)
				{
					Gdiplus::Bitmap* pBitmap = new Gdiplus::Bitmap(pStream);
					if (pBitmap)
					{
						m_pBitmap = pBitmap;
						bSuccess = TRUE;
						return bSuccess;
					}
				}
				::GlobalUnlock(m_hBuffer);
			}
			::GlobalFree(m_hBuffer);
			m_hBuffer = NULL;
		}
		return bSuccess;
	}
private:
	Gdiplus::Bitmap*	m_pBitmap;
	HGLOBAL m_hBuffer;
};